<?php
/**
 * Options View
 *
 * @package    Options View
 * @subpackage OptionsView List Table
 * reference   Custom List Table Example
 *             https://wordpress.org/plugins/custom-list-table-example/
/*
	Copyright (c) 2021- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

if ( ! class_exists( 'WP_List_Table' ) ) {
	require_once( ABSPATH . 'wp-admin/includes/class-wp-screen.php' );
	require_once( ABSPATH . 'wp-admin/includes/screen.php' );
	require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
	require_once( ABSPATH . 'wp-admin/includes/template.php' );
}

/** ==================================================
 * List table
 */
class TT_OptionsView_List_Table extends WP_List_Table {

	/** ==================================================
	 * Max items
	 *
	 * @var $max_items  max_items.
	 */
	public $max_items;

	/** ==================================================
	 * Construct
	 *
	 * @since 2.00
	 */
	public function __construct() {

		global $status, $page;
		/* Set parent defaults */
		parent::__construct(
			array(
				'singular'  => 'bulk_options_update',
				'ajax'      => false,
			)
		);

	}

	/** ==================================================
	 * Read data
	 *
	 * @since 2.00
	 */
	private function read_data() {

		$search_text = get_option( 'optionsview_search_text' );
		if ( isset( $_POST['options-view-filter'] ) && ! empty( $_POST['options-view-filter'] ) ) {
			if ( check_admin_referer( 'opv_filter', 'options_view_filter' ) ) {
				if ( ! empty( $_POST['user_id'] ) ) {
					update_option( 'opv_filter_user', absint( $_POST['user_id'] ) );
				} else {
					update_option( 'opv_filter_user', null );
				}
				if ( ! empty( $_POST['search_text'] ) ) {
					$search_text = sanitize_text_field( wp_unslash( $_POST['search_text'] ) );
					update_option( 'optionsview_search_text', $search_text );
				} else {
					delete_option( 'optionsview_search_text' );
					$search_text = null;
				}
			}
		}

		global $wpdb;
		$listtable_array = array();

		if ( 'options' == get_option( 'options-view', 'options' ) ) {
			$options = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}options ORDER BY option_name" );
			if ( ! empty( $options ) ) {
				foreach ( $options as $option ) {
					$search = false;
					if ( $search_text ) {
						if ( false !== strpos( $option->option_name, $search_text ) ) {
							$search = true;
						}
						if ( false !== strpos( $option->option_id, $search_text ) ) {
							$search = true;
						}
					} else {
						$search = true;
					}

					if ( $search ) {
						$listtable_array[] = array(
							'ID'    => $option->option_id,
							'title' => $option->option_name,
							'ids'   => $option->option_id,
							'edit'  => $option->option_value,
						);
					}
				}
			}
		} else if ( 'user_options' == get_option( 'options-view', 'options' ) ) {
			$user_filter = get_option( 'opv_filter_user' );
			if ( ! empty( $user_filter ) ) {
				$user_options = $wpdb->get_results(
					$wpdb->prepare(
						"
						SELECT *
						FROM {$wpdb->prefix}usermeta
						WHERE user_id = %d
						ORDER BY meta_key
						",
						$user_filter
					)
				);
			} else {
				$user_options = $wpdb->get_results( "SELECT * FROM {$wpdb->prefix}usermeta ORDER BY meta_key" );
			}
			if ( ! empty( $user_options ) ) {
				foreach ( $user_options as $option ) {
					$search = false;
					if ( ! is_null( $search_text ) ) {
						if ( false !== strpos( $option->meta_key, $search_text ) ) {
							$search = true;
						}
						if ( false !== strpos( $option->umeta_id, $search_text ) ) {
							$search = true;
						}
						if ( false !== strpos( $option->user_id, $search_text ) ) {
							$search = true;
						}
					} else {
						$search = true;
					}

					if ( $search ) {
						$listtable_array[] = array(
							'ID'    => $option->umeta_id,
							'title' => $option->meta_key,
							'ids'   => $option->umeta_id,
							'user'  => $option->user_id,
							'edit'  => $option->meta_value,
						);
					}
				}
			}
		}

		return $listtable_array;

	}

	/** ==================================================
	 * Column default
	 *
	 * @param array  $item  item.
	 * @param string $column_name  column_name.
	 * @since 2.00
	 */
	public function column_default( $item, $column_name ) {

		switch ( $column_name ) {
			case 'ids':
				return $item[ $column_name ];
			case 'user':
				return $item[ $column_name ];
			case 'edit':
				$value = maybe_unserialize( $item[ $column_name ] );
				$structure_none = false;
				$structure = '<details>';
				$structure .= '<summary><strong>' . __( 'Structure', 'options-view' ) . '</strong></summary>';
				$structure .= '<pre style="width: 100%; white-space: pre-wrap;">';
				if ( is_array( $value ) || is_object( $value ) ) {
					$structure .= print_r( $value, true );
				} else {
					if ( is_string( $value ) &&
						is_array( json_decode( $value, true ) ) &&
						json_last_error() == JSON_ERROR_NONE ) {
						$value2 = json_decode( $value, true );
						$structure .= print_r( $value2, true );
					} else {
						$structure_none = true;
					}
				}
				$structure .= '</pre>';
				$structure .= '</details>';
				if ( $structure_none ) {
					$structure = null;
				}
				return $structure . sprintf(
					'<textarea name="%1$s[%2$s]" %4$s form="optionsview_forms">%3$s</textarea>',
					/*%1$s*/ $this->_args['singular'],
					/*%2$s*/ $item['ID'],
					/*%3$s*/ $item[ $column_name ],
					/*%4$s*/ 'style="width: 100%;"'
				);
			default:
				return print_r( $item, true ); /* Show the whole array for troubleshooting purposes */
		}
	}

	/** ==================================================
	 * Column title
	 *
	 * @param array $item  item.
	 * @since 2.00
	 */
	public function column_title( $item ) {

		/* Return the title contents */
		return sprintf(
			'%1$s <span style="color:silver"></span>',
			/*$1%s*/ $item['title']
		);
	}

	/** ==================================================
	 * Column checkbox
	 *
	 * @param array $item  item.
	 * @since 2.00
	 */
	public function column_cb( $item ) {

		return sprintf(
			'<input type="checkbox" name="bulk_options_check[]" value="%1$s" form="optionsview_forms" />',
			/*%1$s*/ $item['ID']
		);

	}

	/** ==================================================
	 * Get Columns
	 *
	 * @since 2.00
	 */
	public function get_columns() {

		if ( 'options' == get_option( 'options-view', 'options' ) ) {
			$columns = array(
				'cb'    => '<input type="checkbox" />', /* Render a checkbox instead of text */
				'title' => __( 'Title' ),
				'ids'   => 'ID',
				'edit'  => __( 'Edit' ),
			);
		} else if ( 'user_options' == get_option( 'options-view', 'options' ) ) {
			$columns = array(
				'cb'    => '<input type="checkbox" />', /* Render a checkbox instead of text */
				'title' => __( 'Title' ),
				'ids'   => 'ID',
				'user'  => 'User ID',
				'edit'  => __( 'Edit' ),
			);
		}

		return $columns;

	}

	/** ==================================================
	 * Get Sortable Columns
	 *
	 * @since 2.00
	 */
	public function get_sortable_columns() {
		$sortable_columns = array(
			'title' => array( 'title', false ),
		);
		return $sortable_columns;
	}

	/** ************************************************************************
	 * REQUIRED! This is where you prepare your data for display. This method will
	 * usually be used to query the database, sort and filter the data, and generally
	 * get it ready to be displayed. At a minimum, we should set $this->items and
	 * $this->set_pagination_args(), although the following properties and methods
	 * are frequently interacted with here...
	 *
	 * @global WPDB $wpdb
	 * @uses $this->_column_headers
	 * @uses $this->items
	 * @uses $this->get_columns()
	 * @uses $this->get_sortable_columns()
	 * @uses $this->get_pagenum()
	 * @uses $this->set_pagination_args()
	 **************************************************************************/
	public function prepare_items() {

		/**
		 * First, lets decide how many records per page to show
		 */
		$per_page = get_option( 'opv_per_page', 20 );

		/**
		 * REQUIRED. Now we need to define our column headers. This includes a complete
		 * array of columns to be displayed (slugs & titles), a list of columns
		 * to keep hidden, and a list of columns that are sortable. Each of these
		 * can be defined in another method (as we've done here) before being
		 * used to build the value for our _column_headers property.
		 */
		$columns = $this->get_columns();
		$hidden = array();
		$sortable = $this->get_sortable_columns();

		/**
		 * REQUIRED. Finally, we build an array to be used by the class for column
		 * headers. The $this->_column_headers property takes an array which contains
		 * 3 other arrays. One for all columns, one for hidden columns, and one
		 * for sortable columns.
		 */
		$this->_column_headers = array( $columns, $hidden, $sortable );

		/**
		 * Instead of querying a database, we're going to fetch the example data
		 * property we created for use in this plugin. This makes this example
		 * package slightly different than one you might build on your own. In
		 * this example, we'll be using array manipulation to sort and paginate
		 * our data. In a real-world implementation, you will probably want to
		 * use sort and pagination data to build a custom query instead, as you'll
		 * be able to use your precisely-queried data immediately.
		 */
		$data = $this->read_data();
		do_action( 'opv_filter_form' );

		/**
		 * This checks for sorting input and sorts the data in our array accordingly.
		 *
		 * In a real-world situation involving a database, you would probably want
		 * to handle sorting by passing the 'orderby' and 'order' values directly
		 * to a custom query. The returned data will be pre-sorted, and this array
		 * sorting technique would be unnecessary.
		 *
		 * @param array $a  a.
		 * @param array $b  b.
		 */
		function usort_reorder( $a, $b ) {
			/* If no sort, default to title */
			if ( isset( $_REQUEST['orderby'] ) && ! empty( $_REQUEST['orderby'] ) ) {
				$orderby = sanitize_text_field( wp_unslash( $_REQUEST['orderby'] ) );
			} else {
				$orderby = 'title';
			}
			/* If no order, default to asc */
			if ( isset( $_REQUEST['order'] ) && ! empty( $_REQUEST['order'] ) ) {
				$order = sanitize_text_field( wp_unslash( $_REQUEST['order'] ) );
			} else {
				$order = 'asc';
			}
			$result = strcmp( $a[ $orderby ], $b[ $orderby ] ); /* Determine sort order */
			return ( 'asc' === $order ) ? $result : -$result; /* Send final sort direction to usort */
		}
		usort( $data, 'usort_reorder' );

		/***********************************************************************
		 * ---------------------------------------------------------------------
		 * vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
		 *
		 * In a real-world situation, this is where you would place your query.
		 *
		 * For information on making queries in WordPress, see this Codex entry:
		 * http://codex.wordpress.org/Class_Reference/wpdb
		 *
		 * ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
		 * ---------------------------------------------------------------------
		 */

		/**
		 * REQUIRED for pagination. Let's figure out what page the user is currently
		 * looking at. We'll need this later, so you should always include it in
		 * your own package classes.
		 */
		$current_page = $this->get_pagenum();

		/**
		 * REQUIRED for pagination. Let's check how many items are in our data array.
		 * In real-world use, this would be the total number of items in your database,
		 * without filtering. We'll need this later, so you should always include it
		 * in your own package classes.
		 */
		$total_items = count( $data );
		$this->max_items = $total_items;

		/**
		 * The WP_List_Table class does not handle pagination for us, so we need
		 * to ensure that the data is trimmed to only the current page. We can use
		 * array_slice() to
		 */
		$data = array_slice( $data, ( ( $current_page - 1 ) * $per_page ), $per_page );

		/**
		 * REQUIRED. Now we can add our *sorted* data to the items property, where
		 * it can be used by the rest of the class.
		 */
		$this->items = $data;

		/**
		 * REQUIRED. We also have to register our pagination options & calculations.
		 */
		$this->set_pagination_args(
			array(
				'total_items' => $total_items,                  /* WE have to calculate the total number of items */
				'per_page'    => $per_page,                     /* WE have to determine how many items to show on a page */
				'total_pages' => ceil( $total_items / $per_page ),   /* WE have to calculate the total number of pages */
			)
		);
	}

}


